/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.control
 * File:    IfmapFacade.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.control;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.RemoteException;

import prefuse.Display;
import de.fhhannover.inform.ifmap.datastructure.IdentifierContainer;
import de.fhhannover.inform.ifmap.event.IdentifierChangedReceiver;
import de.fhhannover.inform.ifmap.event.StatusChangedReceiver;
import de.fhhannover.inform.ifmap.exception.PropertiesNotFoundException;
import de.fhhannover.inform.ifmap.output.JPrefuse;
import de.fhhannover.inform.ifmap.properties.PropertiesReader;
import de.fhhannover.inform.ifmap.transform.MessageTransformer;

public final class IfmapFacade {
	private static IfmapFacade ifmap_facade;
	private MessageTransformer transformer;
	private JPrefuse prefuse;
	private String properties;
	private String filter;

	private IfmapFacade() {		
			transformer = new MessageTransformer();		
	}
	
	private IfmapFacade(String path) {
			this();
			this.properties = path;			
	}
	
	public String getPublisherID(){
		return transformer.getPublisherID();
	}
	
	public String getSessionID(){
		return transformer.getSessionID();
	}	
	
	public static IfmapFacade getInstance() {	
			return new IfmapFacade();
	}
	
	public static IfmapFacade getInstance(String propertiespath) {		
			return new IfmapFacade(propertiespath);
	}
	
	public void setPropertiesPath(String props){
		this.properties = props;
	}
	
	public void setDumpFilter(String f){
		filter = f;
	}
	
	public void addStatusListener(StatusChangedReceiver r){
		transformer.addStatusListener(r);
	}
	
	public void addUpdatesListener(IdentifierChangedReceiver r){
		transformer.addUpdateListener(r);
	}
	
	public synchronized void enablePolling(boolean poll){
		transformer.enablePolling(poll);
	}
	
	public synchronized void disconnect(){
		transformer.enablePolling(false);
		transformer.disconnect();
	}
	
	private synchronized void initProperties()
	throws PropertiesNotFoundException{
			if(this.properties == null)
				throw new PropertiesNotFoundException("Properties not set. Use setPropertiesPath() to set path.");				
			try {
				PropertiesReader.loadProperties(properties);
			} catch (FileNotFoundException e) {
				throw new PropertiesNotFoundException("Properties not initialized. File not found: '"+properties+"'");
			} catch (IOException e) {
				throw new PropertiesNotFoundException("Properties not initialized. IOException: '"+properties+"'");
			}
	}
	
	public synchronized IdentifierContainer getSearchIdentifier() 
	throws PropertiesNotFoundException, RemoteException{
		initProperties();
		return transformer.getSearchIdentifier(filter);
	}
	
	public synchronized boolean connect() 
	throws PropertiesNotFoundException, RemoteException{
		initProperties();
		return transformer.connect();
	}
	
	public synchronized boolean subscribe(IdentifierContainer conti) 
	throws PropertiesNotFoundException, RemoteException{
		initProperties();
		//enablePolling(true);
		return transformer.subscribeUpdate(conti);
	}
	
	public synchronized boolean subscribe(Object o) 
	throws PropertiesNotFoundException, RemoteException{
		initProperties();
		//enablePolling(true);
		return transformer.subscribeUpdate(o);
	}
	
	public synchronized boolean isConnected(){
		return (ifmap_facade != null) ? true : false;
	}
	
	public synchronized long getLastUpdateTime() 
	throws PropertiesNotFoundException, RemoteException{
		initProperties();
		return transformer.getLastUpdate();
	}
	
	public Display getPrefuseGraph(){
		if(prefuse == null)
			prefuse = new JPrefuse();
		return prefuse;
	}
}